#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
module Extracter

class Extracter

  # ========================================================================= #
  # === Extracter::Extracter.is_this_a_valid_archive?
  #
  # Query whether the input is a valid archive.
  #
  # The registered formats are stored in the constant
  # ARRAY_REGISTERED_ARCHIVES.
  # ========================================================================= #
  def self.is_this_a_valid_archive?(
      i,
      array_registered_archives = ARRAY_REGISTERED_ARCHIVES
    )
    return_value = false
    array_registered_archives.each {|entry|
      return_value = true if i =~ /#{entry}$/
    }
    return return_value
  end; self.instance_eval { alias is_archive? is_this_a_valid_archive? } # === Extracter::Extracter.is_archive?

  # ========================================================================= #
  # === Extracter::Extracter.extract_what_to
  #
  # This method provides a convenient API to extract something to a 
  # specified directory, as a class method. It defaults to the
  # current working directory, as that is by far the most convenient
  # way to extract a source tarball/archive.
  #
  # Useage example goes like this:
  #
  #   Extracter.extract_what_to('foo-1.0.tar.xz', '/tmp')
  #
  # ========================================================================= #
  def self.extract_what_to(
      what,
      to                 = Dir.pwd, # <- This can also be a Hash.
      optional_be_silent = false
    )
    _ = Extracter.new(what, to, :do_not_run_yet)
    if to.is_a? Hash
      # ===================================================================== #
      # === :prepend_this_namespace
      # ===================================================================== #
      if to.has_key? :prepend_this_namespace
        prefix_with_this = to.delete(:prepend_this_namespace) # Get rid of it from the Hash as well.
        _.prefix_namespace_with(
          prefix_with_this
        )
      end
      # ===================================================================== #
      # === :to
      # ===================================================================== #
      if to.has_key? :to
        to = to.delete(:to)
      end
      # ===================================================================== #
      # === :verbosity
      #
      # Handle how verbose we shall be.
      # ===================================================================== #
      if to.has_key? :verbosity
        _.set_verbosity(
          to.delete(:verbosity)
        )
      end
      # ===================================================================== #
      # === :use_colours
      # ===================================================================== #
      if to.has_key? :use_colours
        _.set_use_colours(to[:use_colours])
      end
      # ===================================================================== #
      # === :pad_opn_with_n_tokens
      # ===================================================================== #
      if to.has_key? :pad_opn_with_n_tokens
        _.set_pad_opn_with_n_tokens(to[:pad_opn_with_n_tokens])
      end
      # ===================================================================== #
      # === :use_opn
      # ===================================================================== #
      if to.has_key? :use_opn
        _.set_use_opn(to[:use_opn])
      end
    end
    if optional_be_silent
      _.be_silent
    else
      _.be_verbose
    end
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :show_the_full_name_of_the_archive
        _.do_show_the_full_name_of_the_archive
      end
    end
    _.run
  end; self.instance_eval { alias extract_this extract_what_to } # === Extracter.extract_this
       self.instance_eval { alias what_to      extract_what_to } # === Extracter.what_to
       self.instance_eval { alias extract      extract_what_to } # === Extracter.extract

  # ========================================================================= #
  # === Extracter::Extracter[]
  # ========================================================================= #
  def self.[](i, where_to = Dir.pwd)
    new(i, where_to)
  end

end

# =========================================================================== #
# === Extracter.is_this_a_valid_archive?
#
# Determine whether the Extracter project can deal with the given archive
# or whether it can not.
# =========================================================================== #
def self.is_this_a_valid_archive?(i)
  ::Extracter::Extracter.is_this_a_valid_archive?(i)
end; self.instance_eval { alias is_archive? is_this_a_valid_archive? } # === Extracter.is_archive?

# =========================================================================== #
# === Extracter.extract_what_to
# =========================================================================== #
def self.extract_what_to(
    what = 'foo-1.0.tar.xz',
    to   = '/tmp/', # This target directory defaults to Linux/UNIX-like systems.
    optional_be_silent = false
  )
  ::Extracter::Extracter.extract_what_to(what, to, optional_be_silent)
end; self.instance_eval { alias what_to extract_what_to } # === Extracter.what_to

# ========================================================================= #
# === Extracter.extract_what_to
# ========================================================================= #
def self.extract_what_to(
    what,
    to                 = Dir.pwd, # <- This can also be a Hash.
    optional_be_silent = false
  )
  ::Extracter::Extracter.extract_what_to(what, to, optional_be_silent)
end; self.instance_eval { alias extract_this extract_what_to } # === Extracter.extract_this
     self.instance_eval { alias what_to      extract_what_to } # === Extracter.what_to
     self.instance_eval { alias extract      extract_what_to } # === Extracter.extract

# =========================================================================== #
# === Extracter[]
# =========================================================================== #
def self.[](i, where_to = Dir.pwd)
  ::Extracter::Extracter.new(i, where_to)
end

end