#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
module Extracter

class Extracter

  begin
    require 'colours'
  rescue LoadError; end

  # ========================================================================= #
  # === use_colours?
  #
  # Determine whether we will use colours in class Extracter.
  # ========================================================================= #
  def use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === set_use_colours
  # ========================================================================= #
  def set_use_colours(i)
    # ======================================================================= #
    # We must also sync this towards our main Hash, for opn(). The next
    # line of code achieves precisely that.
    # ======================================================================= #
    @use_this_opn_hash.update(use_colours: i)
    @use_colours = i
  end

  # ========================================================================= #
  # === disable_colours
  #
  # Use this method if you want to disable colour-support of this class.
  # ========================================================================= #
  def disable_colours
    @use_colours = false
    @colour_to_use_for_directories = ''.dup
  end

  # ========================================================================= #
  # === enable_colours
  # ========================================================================= #
  def enable_colours
    @use_colours = true
    @colour_to_use_for_directories = cyan?
  end

  # ========================================================================= #
  # === set_colour_for_directories
  #
  # Set the colour for directories to use.
  # ========================================================================= #
  def set_colour_for_directories(i)
    @colour_to_use_for_directories = Colours.beautify(i)
  end

  # ========================================================================= #
  # === colour_to_use_for_directories?
  # ========================================================================= #
  def colour_to_use_for_directories?
    if @use_colours
      @colour_to_use_for_directories
    else
      ''
    end
  end

  # ========================================================================= #
  # === cyan
  # ========================================================================= #
  def cyan?
    if @use_colours
      Colours::CYAN
    else
      ''
    end
  end

  # ========================================================================= #
  # === ewarn
  # ========================================================================= #
  def ewarn(i = '')
    if use_colours?
      e Colours.swarn(i)
    else
      e i
    end
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i = '')
    return Colours.simp(i) if @use_colours
    return i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i = '')
    return Colours.sfancy(i) if @use_colours
    return i
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    return Colours.sfancy(i) if @use_colours
    return i
  end

  # ========================================================================= #
  # === sdir
  # ========================================================================= #
  def sdir(i = '')
    return Colours.sdir(i) if @use_colours
    return i
  end

end; end