#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Extracter
#
# The purpose of this class is to abstract extracting source files to
# a target location. You just pass the argument of the file that has
# to be extracted to this class, and it should handle the rest.
#
# We can also "extract" Audio files, by calling the module ExtractAudio.
# You can try this - just pass a .mp4 file as first argument to
# this class.
#
# Usage examples for class Extracter:
#
#   Extracter.new('foo.mp3')
#   Extracter.new('xyz-1.0.tar.gz')
#
# Usage example:
#
#   Extracter.new
#
# =========================================================================== #
# require 'extracter/extracter.rb'
# =========================================================================== #

# =========================================================================== #
# Next, load up .rb files that belong to extracter/.
# =========================================================================== #
require 'extracter/version/version.rb'
require 'extracter/constants/constants.rb'
require 'extracter/class_methods.rb'
require 'extracter/colours.rb'
require 'extracter/do_extract_what_to.rb'
require 'extracter/help.rb'
require 'extracter/initialize.rb'
require 'extracter/misc.rb'
require 'extracter/reset.rb'
require 'extracter/opn.rb'

module Extracter

class Extracter # === Extracter
end

end

if __FILE__ == $PROGRAM_NAME
  # The second argument to Extracter.new is where we will
  # extract to.
  if ARGV.size > 2
    _ = Extracter::Extracter.new(ARGV, :default, false)
  else
    _ = Extracter::Extracter.new(ARGV[0], ARGV[1], false)
  end
  _.extract_to Dir.pwd unless ARGV[1]
  # _.enable_debug
  _.be_verbose
  _.run
end # extracter
    # extracter htop-1.0.2.tar.xz
    # extracter xfig-3.2.5.tar.bz2
    # extract htop* /Depot/
    # extract recode-3.7.tar.xz /Depot/
    # extract qt-4.8.6.tar.xz --to=/home/Temp