#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'extracter/initialize.rb'
# =========================================================================== #
module Extracter

class Extracter

  # ========================================================================= #
  # === initialize
  #
  # The first argument to this method is the archive that should be extracted.
  # This must be a (locally) existing archive, such as foobar.tar.xz or
  # something similar.
  #
  # The second argument to this method, called `to`, specifies the target
  # location, where this class will extract the archive into, if available.
  # Some keywords and shortcuts exist for this option - for instance, TEMP
  # means $MY_TEMP, which can be set by the user.
  #
  # Specific usage example in pure Ruby:
  #
  #   x = Extracter.what_to('pry-0.9.9.4.gem', '/Depot/Temp')
  #
  # ========================================================================= #
  def initialize(
      what        = nil,
      where_to    = nil, # Where to extract into.
      run_already = true,
      &block
    )
    register_sigint
    reset
    if where_to.is_a? Symbol
      if where_to.to_s.include? 'dont' # This also covers :dont_run_yet
        where_to    = nil # This is ok because we expect the user to provide the target location.
        run_already = false
      end
    end
    if debug? # Some debug-information.
      e "The first argument what is: `#{what}`"
      e "The second argument to is:  `#{to}`"
    end
    set_source_location(what)
    set_extract_to(where_to)
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # === Handle Hash input next:
      # ===================================================================== #
      if yielded.is_a? Hash
        if yielded.has_key? :run_already
          run_already = yielded.delete(:run_already)
        end
        # =================================================================== #
        # === :use_colours
        # =================================================================== #
        if yielded.has_key? :use_colours
          set_use_colours yielded.delete(:use_colours)
        end
        # =================================================================== #
        # === :extract_to
        # =================================================================== #
        if yielded.has_key? :extract_to
          set_extract_to yielded.delete(:extract_to)
        end
        # =================================================================== #
        # === :run_simulation
        # =================================================================== #
        if yielded.has_key? :run_simulation
          set_run_simulation(yielded.delete(:run_simulation))
        end
        # =================================================================== #
        # === :use_opn
        # =================================================================== #
        if yielded.has_key? :use_opn
          set_use_opn(yielded.delete(:use_opn))
        end
      else
        case yielded
        when :dont_run_yet,
             :do_not_run_yet
          run_already = false
        end
      end
    end
    case run_already
    when :dont_run_yet,
         :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

end

# =========================================================================== #
# === Extracter.new
# =========================================================================== #
def self.new(
      what        = nil,
      where_to    = nil, # Where to extract into.
      run_already = true,
      &block
    )
  ::Extracter::Extracter.new(what, where_to, run_already, &block)
end

end