#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'extracter/misc.rb'
# =========================================================================== #
module Extracter

class Extracter # === Extracter

  require 'fileutils'

  begin
    require 'multimedia_paradise/audio/extract_audio/extract_audio.rb'
  rescue LoadError; end

  begin
    require 'remove_file_suffix'
  rescue LoadError; end

  # ========================================================================= #
  # Shall we run in simulation mode or not.
  # ========================================================================= #
  attr_accessor :run_simulation

  # ========================================================================= #
  # === debug?
  # ========================================================================= #
  def debug?
    @debug
  end

  # ========================================================================= #
  # === enable_debug
  # ========================================================================= #
  def enable_debug
    @debug = true
  end

  # ========================================================================= #
  # === do_not_show_name
  #
  # Tells us whether to use opn() or not.
  # ========================================================================= #
  def do_not_show_name
    @do_not_show_name = true
  end

  # ========================================================================= #
  # === do_show_name
  #
  # If this is enabled, we will show the name of the file when we invoke
  # copn().
  # ========================================================================= #
  def do_show_name
    @do_not_show_name = false
  end

  # ========================================================================= #
  # === sanitize_input
  # ========================================================================= #
  def sanitize_input(i)
    if i.include? '('
      i.gsub!(/\(/,'\(')
      i.gsub!(/\)/,'\)') if i.include? ')'
      i = pad(i, '"')
    end
    return i
  end

  # ========================================================================= #
  # === check_whether_rar_is_available
  #
  # We try to find out whether unrar is available.
  # ========================================================================= #
  def check_whether_rar_is_available
    is_available = false
    ENV['PATH'].split(':').each {|entry|
      is_available = true if File.exist? entry+'/unrar'
    }
    unless is_available
      copn; e 'Sorry, unrar is not available. Please install it first.'
    end
  end

  # ========================================================================= #
  # === run_simulation?
  # ========================================================================= #
  def run_simulation?
    @run_simulation
  end

  # ========================================================================= #
  # === set_run_simulation
  # ========================================================================= #
  def set_run_simulation(i)
    @run_simulation = i
  end

  # ========================================================================= #
  # === pad
  # ========================================================================= #
  def pad(i, with_this_character = "'")
    return with_this_character+i+with_this_character
  end

  # ========================================================================= #
  # === extract_to?
  #
  # Simply output the result of @extract_to_this_location variable.
  # ========================================================================= #
  def extract_to?
    @extract_to_this_location.to_s
  end; alias source_package_location extract_to? # === source_package

  # ========================================================================= #
  # === set_source_location
  #
  # Use this method to designate the source location of a given (input)
  # program. In other words - the tarball or archive that must be
  # extracted. It will however be stored as Array.
  #
  # If we pass a hash to this method, we assume that the user wants
  # to also populate some other values.
  # ========================================================================= #
  def set_source_location(i = nil)
    if i.nil?
      copn; e 'You should set a file.'
    end
    if i.is_a? Array
      i = i.flatten
    else
      i = [i]
    end # After this point, we will have an Array.
    case i.first # case tag
    when /^-?-?help$/i
      show_help
      exit
    end
    i.map! {|entry| # Iterate over our Array next.
      # ======================================================================= #
      # Handle the case when the user did input a number.
      # ======================================================================= #
      begin
        if entry =~ /^\d$/
          entry = Dir['*'][( entry.to_i - 1 )] unless File.exist?(entry)
        end
      rescue ArgumentError => error
        e 'Error for '+sfancy(entry)+':'
        pp error
      end
      entry = rds(entry.to_s)
      # ======================================================================= #
      # Next, find the proper working directory.
      # ======================================================================= #
      unless entry.include? '/'
        entry = rds( (Dir.pwd+'/'+entry) )
      end
      # ======================================================================= #
      # If the user supplied a directory instead, we will randomly grab an
      # entry from said directory.
      # ======================================================================= #
      if File.directory? entry
        entry = Dir[rds(entry+'/')+'*'].sample
      end
      entry
    } # Sanitize the result, just in case.
    i = [i] unless i.is_a? Array # Much more convenient to work with an array.
    @source_location = i
  end; alias set_what set_source_location # === set_what

  # ========================================================================= #
  # === source_location?
  # ========================================================================= #
  def source_location?
    @source_location.first
  end; alias input? source_location? # === input?

  # ========================================================================= #
  # === did_we_extract_already?
  #
  # Whether we already did extract or whether we did not.
  # ========================================================================= #
  def did_we_extract_already?
    @did_we_extract_already
  end; alias did_we_extract_already did_we_extract_already? # === did_we_extract_already

  # ========================================================================= #
  # === extract_to_this_location?
  #
  # Extract to this location.
  # ========================================================================= #
  def extract_to_this_location?
    @extract_to_this_location
  end; alias extract_to_this_location extract_to_this_location? # === extract_to_this_location

  # ========================================================================= #
  # === extracted_to?
  #
  # This method is different from extract_to?.
  #
  # It will keep track of the directory to where we extracted to
  # exactly.
  # ========================================================================= #
  def extracted_to?
    rds(
      extract_to?+
      File.basename(input?).sub(/\.xz$/,'').sub(/\.gz$/,'').
                            sub(/\.tar$/,'')+'/'
    )
  end; alias extracted_path? extracted_to? # === extracted_path?

  # ========================================================================= #
  # === namespace?
  # ========================================================================= #
  def namespace?
    @namespace
  end

  # ========================================================================= #
  # === set_extract_to_this_location
  #
  # Use this when setting the variable @extract_to_this_location.
  #
  # This can be modified from the commandline such as by doing this:
  # ========================================================================= #
  def set_extract_to_this_location(
      i = TEMP_DIR
    )
    if i.is_a? Hash
      if i.has_key? :to
        i = i.delete :to
      elsif i.has_key? :extract_to
        i = i.delete :extract_to
      end
    end
    case i # case tag
    when :default
      i = Dir.pwd
    when 'TEMP',
         'MY_TEMP',
         'MYTEMP'
      i = TEMP_DIR
    end
    i = TEMP_DIR if i.nil?
    i = i.to_s.dup
    i << '/' unless i.end_with? '/'
    i = rds(i)
    i.gsub!(/--to=/,'') if i.include? '--to='
    @extract_to_this_location = i
  end; alias set_extract_to set_extract_to_this_location # === set_extract_to
       alias extract_to=    set_extract_to_this_location # === extract_to=
       alias extract_to     set_extract_to_this_location # === extract_to

  # ========================================================================= #
  # === run_this_system_command
  # ========================================================================= #
  def run_this_system_command(
      i, instruction = :do_nothing_special
    )
    if instruction == :also_show_what_we_will_do
      copn; e i
    end
    return `#{i}` # system tag
  end

  # ========================================================================= #
  # === prefix_namespace_with
  # ========================================================================= #
  def prefix_namespace_with(i)
    @namespace = "#{i}#{@namespace.dup}"
    update_the_opn_hash # Also update the opn-hash here.
  end

  # ========================================================================= #
  # === report_to_the_user
  #
  # This method reports to the user. Usually this is done only via this
  # file here though.
  # ========================================================================= #
  def report_to_the_user
    if @be_verbose
      unless @skip_extracting
        copn; e 'Finished extracting to `'+sdir(
          extract_to?+remove_file_extension(
            @source_location.first # This is an Array.
          )+'/'
        )+'`.'
      end
    end
  end

  # ========================================================================= #
  # === be_verbose?
  #
  # Getter method for whether we will be verbose or not.
  # ========================================================================= #
  def be_verbose?
    @be_verbose
  end

  # ========================================================================= #
  # === be_verbose
  # ========================================================================= #
  def be_verbose
    set_be_verbose(true)
  end

  # ========================================================================= #
  # === be_silent
  # ========================================================================= #
  def be_silent
    set_be_verbose(false)
  end

  # ========================================================================= #
  # === set_be_verbose
  #
  # This sets the verbosity level of the class. Use only this method
  # when you wish to modify the @be_verbose instance variable.
  # ========================================================================= #
  def set_be_verbose(i = false)
    @be_verbose = i
  end; alias set_verbosity set_be_verbose # === set_verbosity

  # ========================================================================= #
  # === determine_default_opn_hash
  # ========================================================================= #
  def determine_default_opn_hash
    @use_this_opn_hash = {
      namespace:   namespace?,
      use_colours: use_colours?
    }
  end; alias update_the_opn_hash determine_default_opn_hash # === update_the_opn_hash

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint
    Signal.trap('SIGINT') {
      e sfancy('Requesting a graceful exit from ')+
        colour_to_use_for_directories?+
       'class Extracter'+
       sfancy('. Exiting now.')
      exit
    }
  end

  # ========================================================================= #
  # === create_directory
  #
  # Use this to create directories.
  # ========================================================================= #
  def create_directory(i)
    FileUtils.mkdir_p(i)
  end; alias mkdir create_directory # === mkdir

  # ========================================================================= #
  # === remove_file_extension
  # ========================================================================= #
  def remove_file_extension(i)
    _ = File.basename(i)
    return RemoveFileSuffix[_]
  end

  # ========================================================================= #
  # === rds
  # ========================================================================= #
  def rds(i)
    i.squeeze('/')
  end

  # ========================================================================= #
  # === esystem
  # ========================================================================= #
  def esystem(i)
    e i
    system i
  end

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    puts i
  end

  # ========================================================================= #
  # === do_show_the_full_name_of_the_archive
  # ========================================================================= #
  def do_show_the_full_name_of_the_archive
    @show_only_the_short_name_of_the_archive = false
  end

  # ========================================================================= #
  # === fail_message_not_registered
  #
  # Output a fail message when the archive format is not registered. 
  # ========================================================================= #
  def fail_message_not_registered(i)
    copn; e "Can not extract `#{sfancy(i)}` - it is not "\
            "registered."
  end

  # ========================================================================= #
  # === work_on_the_given_input
  # ========================================================================= #
  def work_on_the_given_input
    if @source_location.empty?
      copn; e 'Can not extract anything as no input has been given.'
    else
      @source_location.each {|entry|
        if Extracter.is_this_a_valid_archive?(entry)
          do_extract_what_to(entry)
          report_to_the_user
        else
          fail_message_not_registered(entry)
        end
      }
    end
  end

  # ========================================================================= #
  # === run                                                (run tag, def tag)
  # ========================================================================= #
  def run
    work_on_the_given_input
  end

end; end